#
# Christmas Logic 
# Copyright (C) Jack Whitham 2009
#

import os, sys
import cPickle as pickle
import network, logic, cso, baselevel
from constants import *
from disk import RO_DIR, RW_DIR

class Level_Container:
    def __init__(self):
        self.number = None
        self.path = None
        self.name = ""
        self.completed = False
        self.is_unreal = False
        self.Reset()

    def Reset(self):
        self.network = network.Network()
        self.strings = []
        self.desc = ""
        self.nopickle = Level_Container_Nopickle()
        self.guide_component = None
        self.guide_data = []
        self.dis_inputs = False
       
class Level_Container_Nopickle:
    def __init__(self):
        self.image = None

class Game_Container:
    def __init__(self):
        self.game_db_file = os.path.join(RW_DIR(), GAME_DB_FILE)
        try:
            self.game_db = pickle.load(file(self.game_db_file, "rb"))
        except:
            self.game_db = None

        if ((self.game_db == None)
        or (self.game_db.get("game_version", "") != VERSION)
        or not os.path.isdir(self.game_db[ "levelpath" ])):
            self.__Initialise()
        else:
            self.__Refresh()
        self.Save()
        print 'Ready'

    def Save(self):
        swapout = []
        for level in self.game_db[ "levels" ]:
            assert level.nopickle != None
            swapout.append(level.nopickle)
            level.nopickle = None

        pickle.dump(self.game_db, file(self.game_db_file, "wb"))

        for (level, nopickle) in zip(self.game_db[ "levels" ], swapout):
            level.nopickle = nopickle
            assert level.nopickle != None
        print 'Database saved'
        
    def Get_Level_Container(self, number, force_reset):
        level = self.game_db[ "levels" ][ number ]
        if force_reset:
            self.__Load_Level(level)
        return level

    def Get_Number_of_Levels(self):
        return len(self.game_db[ "levels" ])

    def __Initialise(self):
        levels = []
        self.game_db = {
            "game_version" : VERSION,
            "levelpath" : os.path.abspath(os.path.join(RO_DIR(), LEVEL_PATH)),
            "levels" : levels,
            }

        lev_list = []
        for fname in os.listdir(os.path.join(RO_DIR(), LEVEL_PATH)):
            fname = os.path.abspath(os.path.join(RO_DIR(), 
                        LEVEL_PATH, fname))
            if (os.path.isfile(fname)
            and fname.endswith(".py")):
                lev_list.append(fname)
       
        lev_list.sort()
        print "Initialising database, please wait..."
        for (i, path) in enumerate(lev_list):
            level = Level_Container()
            level.number = i
            level.path = path
            self.__Load_Level(level)
            levels.append(level)
            print "Initialised level %d" % i

    def __Refresh(self):
        for level in self.game_db[ "levels" ]:
            n = level.network
            assert n != None
            self.__Load_Level(level)
            level.network = n
            level.guide_component = level.network.Get_Guide_Component()
            if level.guide_component != None:
                level.guide_component.Reset()

    def __Load_Level(self, level):
        assert level != None

        level.Reset()
        
        right = []
        left = []
        band_pos = {
                0 : (0, 100),
                1 : (250, 70),
                2 : (500, 150),
            }

        def Meta_Data(text, desc, 
                    dis_inputs = False, guide = None, 
                    image = None, tick = 1.0, unreal_level = False):
            if guide == None:
                guide = []

            level.dis_inputs = dis_inputs
            level.is_unreal = unreal_level
            level.name = text
            level.desc = desc
            level.guide_data = guide
            level.nopickle.image = cso.CSO_Image(
                        level.strings, image)
            
        def Create_Component(create_class, *args, **args2):
            c = create_class(level.network, *args, **args2)
            if isinstance(c, logic.Light_Control):
                band = 2
                right.append(c)
            elif isinstance(c, logic.Stimulator):
                band = 0
                left.append(c)
            else:
                band = 1
        
            (x, y) = band_pos[ band ]
            c.Set_Location((x, y))
            y += 70
            band_pos[ band ] = (x, y)
            return c

        def Gap():
            band = 1
            (x, y) = band_pos[ band ]
            y += 70
            band_pos[ band ] = (x, y)

        def Move_Right(dist):
            for c in right:
                (x, y) = c.Get_Location()
                x += dist
                c.Set_Location((x, y))

        def New_Column():
            Move_Right(150)
            (x, y) = band_pos[ 1 ]
            x += 150
            band_pos[ 1 ] = (x, 70)

        def Create_String(create_class, *args, **args2):
            s = create_class(*args, **args2)
            level.strings.append(s)
            return s

        level_vars = {
                "Meta_Data"         : Meta_Data,
                "Create_Component"  : Create_Component,
                "Create_String"     : Create_String,
                "Gap"               : Gap,
                "New_Column"        : New_Column,
            }
        execfile(level.path, level_vars)

        for k in level_vars.keys():
            v = level_vars[ k ]
            if (k.startswith("_")
            or (type(v) in [type(logic),            # module
                           type(Exception),         # exception
                           type(Game_Container),    # class
                           type(Gap)])):            # method
                # Not pickleable so discard
                del level_vars[ k ]

        if len(left) == 0:
            # For levels with no inputs
            Move_Right(250)

        if len(level.guide_data) > 0:
            (x, y) = band_pos[ 0 ]
            (x1, y1) = band_pos[ 1 ]
            y = max(y, y1, 300) + 100
            c = logic.Guide_Component(level.network, 
                        level.guide_data, level_vars)
            c.Set_Location((GUIDE_BOX_WIDTH / 2, y))
            c.Reset()
            level.guide_component = c

        level.network.Tick()



