#
# Christmas Logic 
# Copyright (C) Jack Whitham 2009
#

import wx
import font, dfl, cso
from constants import *
from disk import RO_DIR

NUM_OVERLAYS = 6
INTERVAL = 4
END_SPLASH = ("end", "splash")
CREDITS_OFFSET = 50
CREDITS = [
    '"Christmas Logic" code copyright (C) Jack Whitham 2009 '
            '- www.jwhitham.org.uk',
    '"Santa" drawings copyright (C) Neill Cameron 2008 - www.neillcameron.com',
    '"Christmas Tree" from Open Clipart (christmas_tree_mo_01.svg) -',
    '    www.clipart.clipartist.net/openclipart',
    'Other artwork by Jack Whitham based on product photographs',
    '    from Blue World Gardener - www.blueworldgardener.co.uk',
    '  ',
    'For Jilly',
    'Made with wxPython and py2exe and released as free software '
            'under GNU GPLv2',
]

class Guide:
    def __init__(self):
        self.FONT_GUIDE = font.Make_Font(wx.FONTFAMILY_ROMAN, 20)
        self.PEN_REGULAR = wx.Pen((0, 0, 0), 1, wx.SOLID)
        self.PEN_ARROW = wx.Pen((200, 0, 0), 5, wx.SOLID)
        self.BR_GUIDE = wx.Brush((200, 200, 255))
        self.contents = None
        self.bitmap = None
        self.point_at = None
        self.location = (0, 0)
        self.size = (1, 1)
        self.arrow_line = []
        self.pointer = 0


        fake_level_strings = []
        binary = { 0 : 64 }
        import baselevel
        for i in xrange(1, NUM_OVERLAYS + 1, 1):
            fake_level_strings.append(baselevel.String(
                    name = "z%d" % i,
                    colour = (binary.get(i & 1, 192),
                            binary.get(i & 2, 160),
                            binary.get(i & 4, 192)),
                    sequence = "0"))

        self.cso_image = cso.CSO_Image(fake_level_strings, "splash")
        self.splash_bitmap = wx.Bitmap(
                    os.path.join(RO_DIR(), IMAGE_PATH, "splash.png"))

        h = 0
        w = 0
        dc = wx.MemoryDC()
        dc.SetFont(self.FONT_GUIDE)
        for text in CREDITS:
            (x, y) = dc.GetTextExtent(text)
            w = max(w, x) 
            h += y
        self.credits_bitmap = cso.Empty_Bitmap((w, h), (255, 255, 255))
        dc.SelectObject(self.credits_bitmap)
        dc.SetTextForeground((0, 0, 0))
        y = 0
        for text in CREDITS:
            dc.DrawText(text, 0, y)
            (_, h) = dc.GetTextExtent(text)
            y += h

        dc.SelectObject(wx.NullBitmap)

    def __Update_Rect(self, logical):
        (x, y) = self.location
        (w, h) = self.size
        x -= w / 2
        y -= h / 2
        if self.contents in END_SPLASH:
            (w1, h1) = self.credits_bitmap.GetSize()
            h += h1 + CREDITS_OFFSET
        logical.Update_Area((x, y), (w, h))
        
    def __Update_Arrow(self, logical):
        m = 5   # Margin to account for width of line
        m2 = m * 2
        for (x1, y1, x2, y2) in self.arrow_line:
            if x2 < x1: (x1, x2) = (x2, x1)
            if y2 < y1: (y1, y2) = (y2, y1)
            logical.Update_Area((x1 - m, y1 - m), (m2 + x2 - x1, m2 + y2 - y1))

    def Setup(self, logical, location, requires_ack = False,
                contents = None, point_at = None, ack_text = None):

        if contents == None:
            point_at = None

        relocate = ((contents != self.contents)
                or (location != self.location))

        if relocate:
            # Change of contents or location (refresh old location)
            relocate = True
            self.__Update_Rect(logical)

        if ((point_at != self.point_at)
        or (location != self.location)):
            # Change of arrow (refresh old arrow)
            self.__Update_Arrow(logical)

            # Compute new arrow
            p1 = (x1, y1) = location
            d12 = d12a = 0
            self.arrow_line = []

            if point_at != None:
                p2 = (x2, y2) = point_at
                d12 = dfl.Distance(p1, p2)
                d12a = d12 - ARROW_OFFSET

            if d12a > 1:

                # Offset the point of the arrow slightly
                x2 = int(x1 + (((x2 - x1) * d12a) / d12))
                y2 = int(y1 + (((y2 - y1) * d12a) / d12))
                d12 = d12a

                # Compute location of the arrow head
                d13 = d12 - ARROW_SIZE
                x3 = int(x1 + (((x2 - x1) * d13) / d12))
                y3 = int(y1 + (((y2 - y1) * d13) / d12))
                x4 = int(x3 + (y2 - y3))
                y4 = int(y3 - (x2 - x3))
                x5 = int(x3 - (y2 - y3))
                y5 = int(y3 + (x2 - x3))

                # Plan location of arrow
                self.arrow_line.append((x1, y1, x2, y2))
                self.arrow_line.append((x4, y4, x2, y2))
                self.arrow_line.append((x5, y5, x2, y2))
                
            # Change of arrow (refresh new arrow)
            self.__Update_Arrow(logical)

        if ((contents != self.contents)
        and (contents == None)):
            # Change of contents to None
            self.contents = contents
            self.bitmap = None
            self.size = (1, 1)

        elif ((contents != self.contents)
        and (contents in END_SPLASH)):
            # Change of contents to Splash picture
            self.contents = contents
            self.bitmap = self.splash_bitmap
            self.size = self.bitmap.GetSize()

        elif contents != self.contents:
            # Change of contents
            self.contents = contents

            #print 'Update guide contents'
            split_settings = []
            dc = wx.MemoryDC()
            dc.SetFont(self.FONT_GUIDE)
            x = GUIDE_BOX_MARGIN
            y = GUIDE_BOX_MARGIN
            overall_h = GUIDE_BOX_HEIGHT 
            limit = GUIDE_BOX_WIDTH - GUIDE_BOX_MARGIN
            (space, h) = dc.GetTextExtent(" ")
            for word in self.contents.split():
                (w, h) = dc.GetTextExtent(word)
                cont = False
                if (w + x) > limit:
                    # Word wrap
                    y += h
                    x = GUIDE_BOX_MARGIN
                    overall_h = max(overall_h, y + h)

                split_settings.append((x, y, word))
                
                x += w + space
    
            bmp = wx.EmptyBitmap(GUIDE_BOX_WIDTH, overall_h)
            dc.SelectObject(bmp)
            dc.SetPen(self.PEN_REGULAR)
            dc.SetBrush(self.BR_GUIDE)
            dc.DrawRectangle(0, 0, GUIDE_BOX_WIDTH, overall_h)

            # Draw the text within the guide box
            dc.SetTextForeground((0, 0, 0))
            for (x, y, word) in split_settings:
                dc.DrawText(word, x, y)

            dc.SelectObject(wx.NullBitmap)
            self.bitmap = bmp
            self.size = (GUIDE_BOX_WIDTH, overall_h)

        if relocate:
            # Change of contents or location (refresh new location)
            self.location = location
            self.point_at = point_at
            self.__Update_Rect(logical)

        if contents == None:
            return None
        elif self.contents in END_SPLASH:
            # Auto-update
            self.pointer += 1
            if (self.pointer % INTERVAL) == 0:
                self.__Update_Rect(logical)

        (w, h) = self.size
        (x, y) = self.location
        x -= w / 2
        y += h / 2
        y += 5
        return (x, y, w)


    def Draw(self, dc):
        dc.SetPen(self.PEN_ARROW)
        for (x1, y1, x2, y2) in self.arrow_line:
            dc.DrawLine(x1, y1, x2, y2)
    
        if self.bitmap != None:
            (x, y) = self.location
            (w, h) = self.bitmap.GetSize()
            x = x - (w / 2)
            y = y - (h / 2)
            dc.DrawBitmap(self.bitmap, x, y)

            if self.contents in END_SPLASH:
                if self.contents == "end":
                    i = (self.pointer / INTERVAL) % NUM_OVERLAYS
                    dc.DrawBitmap(
                        self.cso_image.Get_Overlay(i, ST_ONE), 
                        x, y, True)

                # Draw credits
                y += h + CREDITS_OFFSET
                dc.DrawBitmap(self.credits_bitmap, x, y)

